/*
    Proxy Switch
        あかつきみさき

    このスクリプトについて
        プロキシの有効・無効を切り替えるスクリプト.

    オンオフの適用範囲
        ・All
            プロジェクト内のすべてのプロキシ
        ・Comp
            アクティブなコンポジションとその中に含まれるすべてのプロキシ
        ・Select
            プロジェクトパネル上で選択したプロキシ

    使用方法
        1.ファイル→スクリプト→スクリプトファイルの実行から実行.
        2.ScriptUIフォルダに入れてから,ウィンドウより実行.

    動作環境
        Adobe After Effects CS6以上

    バージョン情報
        2020/09/21 Ver 1.0.0 Release
*/

/// <reference path="D:\lib\aftereffects.d.ts\ae.d.ts"/>

(function() {
    var data = {
        scriptName: "Proxy Switch",
        scriptVersion: "1.0.0"
    };

    Settings.prototype.initSetting = function(sectionName, keyName, defauleValue) {
        if (!app.settings.haveSetting(sectionName, keyName)) {
            app.settings.saveSetting(sectionName, keyName, defauleValue);
        }

        return app.settings.getSetting(sectionName, keyName);
    }

    Project.prototype.enabledProxy = function(value) {
        for (var i = 1; i <= app.project.items.length; i++) {
            if (app.project.item(i).proxySource == null) {
                continue;
            }

            app.project.item(i).useProxy = value;
        }
    }

    CompItem.prototype.enableProxy = function(value, compIds) {
        for (var i = 0; i < compIds.length; i++) {
            if (Number(this.id) == compIds[i]) {
                return 0;
            }
        }

        for (var i = 1; i <= this.layers.length; i++) {
            if (this.layer(i) instanceof ShapeLayer) {
                continue;
            }

            if (this.layer(i).source == null) {
                continue;
            }

            if (this.layer(i).source instanceof CompItem) {
                this.layer(i).source.enableProxy(value, compIds);
            }

            if (this.layer(i).source.proxySource == null) {
                continue;
            }

            this.layer(i).source.useProxy = value;
        }

        compIds[compIds.length] = Number(this.id);
    }

    Project.prototype.enableActiveCompProxy = function(value) {
        var actComp = this.activeItem;

        if (actComp && actComp instanceof CompItem) {
            if (actComp.proxySource != null) {
                actComp.useProxy = value;
            }

            actComp.enableProxy(value, []);
        }
    }

    Project.prototype.enableSelectProxy = function(value) {
        var compIds = [];
        var actComp = this.activeItem;

        if (actComp && actComp instanceof CompItem) {
            var selLayers = actComp.selectedLayers;
            if (selLayers.length == 0) {
                if (actComp.proxySource == null) {
                    return 0;
                }

                actComp.useProxy = value;
                return 0;
            }

            for (var i = 0; i < selLayers.length; i++) {
                if (selLayers[i] instanceof ShapeLayer) {
                    continue;
                }

                if (selLayers[i].source == null) {
                    continue;
                }

                if (selLayers[i].source instanceof CompItem) {
                    selLayers[i].source.enableProxy(value, compIds);
                }

                if (selLayers[i].source.proxySource == null) {
                    continue;
                }

                selLayers[i].source.useProxy = value;
            }
        } else {
            for (var i = 1; i <= this.items.length; i++) {
                if (!this.item(i).selected) {
                    continue;
                }

                if (this.item(i).proxySource == null) {
                    continue;
                }

                this.item(i).useProxy = value;
            }
        }
    }

    function buildUI(thisObj) {
        var emWin;
        if (thisObj instanceof Panel) {
            emWin = thisObj;
        } else {
            emWin = new Window("palette", data.scriptName, undefined, {
                resizeable: true
            });
        }

        emWin.orientation = "column";
        emWin.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];
        emWin.margins = [5, 5, 5, 5];
        emWin.spacing = 5;

        var mainGrp = emWin.add("group");
        mainGrp.margins = [0, 0, 0, 0];
        mainGrp.spacing = 5;
        mainGrp.orientation = "column";
        mainGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.TOP];

        var rbGrp = mainGrp.add("group");
        rbGrp.margins = [0, 0, 0, 0];
        rbGrp.spacing = 0;
        rbGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        rbGrp.orientation = "row";

        var allRb = rbGrp.add("radiobutton", undefined, "All");
        allRb.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        allRb.helpTip = "Switch all proxy";
        allRb.value = true;

        var compRb = rbGrp.add("radiobutton", undefined, "Comp");
        compRb.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        compRb.helpTip = "Switch proxy of active composition tree";

        var selectRb = rbGrp.add("radiobutton", undefined, "Select");
        selectRb.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        selectRb.helpTip = "Switch proxy of selected composition";

        var footerGrp = mainGrp.add("group");
        footerGrp.margins = [0, 0, 0, 0];
        footerGrp.spacing = 0;
        footerGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        footerGrp.orientation = "row";

        var onBtn = footerGrp.add("button");
        onBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        onBtn.text = "ON";

        var offBtn = footerGrp.add("button");
        offBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
        offBtn.text = "OFF";

        var helpBtn = footerGrp.add("button");
        helpBtn.alignment = [ScriptUI.Alignment.RIGHT, ScriptUI.Alignment.FILL];
        helpBtn.text = "?";
        helpBtn.minimumSize.width = helpBtn.maximumSize.width = 25;

        onBtn.onClick = function() {
            app.beginUndoGroup("Proxy ON");
            if (allRb.value) {
                app.project.enabledProxy(true);
            } else if (compRb.value) {
                app.project.enableActiveCompProxy(true);
            } else if (selectRb.value) {
                app.project.enableSelectProxy(true);
            }
            app.endUndoGroup();
        }

        offBtn.onClick = function() {
            app.beginUndoGroup("Proxy OFF");
            if (allRb.value) {
                app.project.enabledProxy(false);
            } else if (compRb.value) {
                app.project.enableActiveCompProxy(false);
            } else if (selectRb.value) {
                app.project.enableSelectProxy(false);
            }
            app.endUndoGroup();
        }

        helpBtn.onClick = function() {
            alert(data.scriptName + " " + data.scriptVersion + "\nあかつきみさき\nhttp://sunrisemoon.net/", data.scriptName + " " + data.scriptVersion);
        }

        emWin.layout.layout();
        emWin.layout.resize();

        emWin.onResizing = emWin.onResize = function() {
            emWin.layout.resize();
            return 0;
        }

        if (!(emWin instanceof Panel)) {
            emWin.center();
            emWin.show();
            return 0;
        }
    }

    buildUI(this);

    return 0;
}).call(this);